/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-chat-server/src/org/opendj/server/chat/ChatSocket.java,v 1.1.1.1 2001/09/29 20:21:13 jonathan Exp $
 * $Log: ChatSocket.java,v $
 * Revision 1.1.1.1  2001/09/29 20:21:13  jonathan
 * Imported opendj-chat-server
 *
 */
package org.opendj.server.chat;

import java.net.Socket;
import org.opendj.util.Util;

public class ChatSocket implements Runnable {

    public static final char SOCKDATA_DELIM  = '^';

    private ChatConnection   itsChatConnection = null;
    private Socket           itsSocket         = null;
    private ChatSocketReader itsSocketReader   = null;
    private ChatSocketWriter itsSocketWriter   = null;
    private volatile boolean isDone = false;

    public ChatSocket ( ChatConnection theConn, 
                        Socket theSocket ) {
        itsChatConnection = theConn;
        itsSocket = theSocket;
    }

    public void run () {
        isDone = false;

        // Setup and start ChatSocketReader
        itsSocketReader = new ChatSocketReader(itsChatConnection, this);
        Thread aSockReaderThread = new Thread(itsSocketReader);
        aSockReaderThread.start();

        // Setup ChatSocketWriter
        itsSocketWriter = new ChatSocketWriter(this);
        
        // Hand off control to writer
        itsSocketWriter.run();
    }

    public void log ( String s ) {
        itsChatConnection.log("[CHAT-SOCKET] " + s);
    }

    public void log ( String s, Exception e ) {
        itsChatConnection.log("[CHAT-SOCKET] " + s + ": " + e.toString() + "\n" + Util.getStackTrace(e));
    }

    public synchronized void finish () {
        isDone = true;
        if ( itsSocketReader != null ) {
            itsSocketReader.finish();
            itsSocketReader = null;
        }
        if ( itsSocketWriter != null ) {
            itsSocketWriter.finish();
            itsSocketWriter = null;
        }
        itsChatConnection.finish();
    }

    public Socket getSocket() { return itsSocket; }

    public int getReaderListenPort() { 
        if ( itsSocketReader != null ) {
            return itsSocketReader.getServerPort(); 
        }
        return -1;
    }

    public boolean getIsReaderReady() {
        return itsSocketReader.getIsReady();
    }

    public void sendData ( String theCommand ) {
        sendData( theCommand, null, null );
    }

    public void sendData ( String theCommand, 
                           String theMessage ) {
        sendData( theCommand, theMessage, null );
    }
    
    public void sendData ( String theCommand,
                           String theData,
                           String theSource ) {

        itsSocketWriter.sendData(theCommand, theData, theSource);
    }

    public boolean getIsDone() { return isDone; }
}
