/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-chat-server/src/org/opendj/server/chat/ChatSocketWriter.java,v 1.1.1.1 2001/09/29 20:21:13 jonathan Exp $
 * $Log: ChatSocketWriter.java,v $
 * Revision 1.1.1.1  2001/09/29 20:21:13  jonathan
 * Imported opendj-chat-server
 *
 */
package org.opendj.server.chat;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.SocketException;
import org.opendj.util.Util;
import org.opendj.util.Config;
import org.opendj.util.BetterTokenizer;

public class ChatSocketWriter implements Runnable {

    private ChatSocket     itsChatSocket     = null;
    private Socket         itsSocket         = null;
    private PrintWriter    itsSockWriter     = null;

    private volatile Thread itsMainThread = null;
    private volatile boolean isDone = false;

    public ChatSocketWriter ( ChatSocket theChatSocket ) {
        itsChatSocket = theChatSocket;
    }

    public void run () {

        itsMainThread = Thread.currentThread();
        isDone = false;

        try {
            itsSocket = itsChatSocket.getSocket();
            // itsSocket.setSoTimeout(Protocol.SOCKET_TIMEOUT_MILLIS);
            itsSocket.shutdownInput();
            itsSockWriter = new PrintWriter( new OutputStreamWriter( itsSocket.getOutputStream() ),
                                             true ); // with autoflush
            log("Created ChatSocketWriter for remote IP " + itsSocket.getInetAddress());

            // Wait until the ChatSocketReader is ready
            int tryCount = 0;
            while ( !itsChatSocket.getIsReaderReady() ) {
                Util.sleep(100);
                tryCount++;
                if ( tryCount > 25 ) {
                    log("Error setting up ChatSocketReader, aborting connection");
                    return;
                }
            }

            // Wait another 250 ms, then send the CONFIG message to the
            // client
            Util.sleep(250);
            int aServerPort = itsChatSocket.getReaderListenPort();
            if ( aServerPort == -1 ) {
                log("ChatSocketReader said it was ready, but port was -1");
                return;
            }
            sendData(Protocol.RES_CONFIG, String.valueOf(aServerPort), null);

            // Now just sit and wait for the ChatConnection to send things
            // our way via the sendData methods
            while ( !isDone ) {
                log("in main loop...");
                if ( itsChatSocket.getIsDone() ) {
                    log("ChatSocket is done, exiting.");
                    return;
                }
                Util.sleep(Protocol.PING_MILLIS);
                if ( isDone ) break;
                sendData(Protocol.RES_PING, null, null);
            }

        } catch ( Exception e ) {
            log("Exception in run", e);

        } finally {
            log("ChatSocketWriter closing from finally clause in run...");
            isDone = true;
            try { itsSocket.close(); } catch ( Exception e ) {}
            itsChatSocket.finish();
        }
    }

    public synchronized void sendData ( String theCommand,
                                        String theData,
                                        String theSource ) {

        log("ChatSocketWriter.sendData('" + theCommand + "')...");

        if ( theData == null ) theData = "";
        if ( theSource  == null ) theSource  = "";

        String someData = theCommand
            + ChatSocket.SOCKDATA_DELIM
            + theData
            + ChatSocket.SOCKDATA_DELIM
            + theSource
            ;

        try {
            itsSockWriter.println(someData);
            log(">" + someData);
        } catch ( Exception e ) {
            log("ChatSocketWriter.sendData, error sending '" + someData + "'", e);
        }
    }

    public void log ( String s ) {
        itsChatSocket.log("[Writer] " + s);
    }

    public void log ( String s, Exception e ) {
        itsChatSocket.log("[Writer] " + s + ": " + e.toString() + "\n" + Util.getStackTrace(e));
    }

    public void finish () {
        isDone = true;
        if ( itsMainThread != null ) {
            itsMainThread.interrupt();
        }
    }

    public boolean getIsDone() { return isDone; }
}
