/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-chat-server/src/org/opendj/server/chat/PStore.java,v 1.1.1.1 2001/09/29 20:21:13 jonathan Exp $
 * $Log: PStore.java,v $
 * Revision 1.1.1.1  2001/09/29 20:21:13  jonathan
 * Imported opendj-chat-server
 *
 */
package org.opendj.server.chat;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;
import java.util.HashMap;
import org.opendj.beans.dj.DjBean;
import org.opendj.beans.dj.NoSuchUserException;
import org.opendj.beans.dj.BadPasswordException;
import org.opendj.util.Util;
import org.opendj.util.DBUtil;

public class PStore {

    private ChatServer itsServer = null;
    private String   itsDBURL      = null;
    private String   itsDBUser     = null;
    private String   itsDBPassword = null;

    public PStore ( ChatServer theServer, 
                    String theDBURL, 
                    String theDBUser, 
                    String theDBPassword ) {
        itsServer     = theServer;
        itsDBURL      = theDBURL;
        itsDBUser     = theDBUser;
        itsDBPassword = theDBPassword;
    }

    public Map getChatRooms (ChatServer theServer) throws SQLException {

        Connection aConn = null;
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT * FROM ChatRoom";
        ChatRoom aRoom = null;
        Map aRoomMap = new HashMap();

        try {
            aConn = getConnection();
            aPS = aConn.prepareStatement(aSQL);
            aRS = aPS.executeQuery();
            while ( aRS.next() ) {
                aRoom = new ChatRoom(theServer);
                aRoom.init(aRS);
                aRoomMap.put( aRoom.getRoomname(), aRoom );
            }
            return aRoomMap;

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    public DjBean loginDj ( String theEmail, 
                            String thePassword ) 
        throws SQLException, NoSuchUserException, BadPasswordException {

        Connection aConn = null;
        DjBean aDjBean = new DjBean();
        int aStatus = -1;
        try {
            aConn = getConnection();
            aStatus = aDjBean.login( aConn, theEmail, thePassword );
            switch ( aStatus ) {
            case DjBean.LOGIN_OK:
                return aDjBean;
            case DjBean.LOGIN_NOSUCHUSER:
                throw new NoSuchUserException();
            case DjBean.LOGIN_BADPASSWORD:
                throw new BadPasswordException();
            default:
                return null;
            }
        } finally {
            DBUtil.closeConnection(aConn);
        }
    }

    public void refreshChatRoom( ChatRoom theRoom ) throws SQLException {

        Connection aConn = null;
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT * FROM ChatRoom WHERE roomname = ?";

        try {
            aConn = getConnection();
            aPS = aConn.prepareStatement(aSQL);
            aPS.setString(1, theRoom.getRoomname());
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                theRoom.init(aRS);
            } else {
                throw new SQLException("No chat room exists with name '" 
                                       + theRoom.getRoomname() + "'");
            }

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    public boolean checkDJNameExists ( String theDJName ) throws SQLException {

        Connection aConn = null;
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL = "SELECT email FROM Dj WHERE Djname = ? ";
        try {
            aConn = getConnection();
            aPS = aConn.prepareStatement(aSQL);
            aPS.setString(1, theDJName);
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                return true;
            }
            return false;

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    public void addBootRecord ( int theModeratordjid, 
                                String theBootedUser, 
                                String theBootedIPAddress ) throws SQLException {
        Connection aConn = null;
        PreparedStatement aPS = null;
        String aSQL
            = "INSERT INTO ChatBoot ( djid, boottime, ipaddr, name ) "
            + "VALUES ( ?, ?, ?, ? ) ";
        try {
            aConn = getConnection();
            aPS = aConn.prepareStatement(aSQL);
            int i=1;
            aPS.setInt   (i++, theModeratordjid);
            aPS.setLong  (i++, System.currentTimeMillis());
            aPS.setString(i++, theBootedIPAddress);
            aPS.setString(i++, theBootedUser);
            aPS.executeQuery();
        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    public boolean isBlockedIP ( String theIP ) throws SQLException {

        Connection aConn = null;
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT name "
            + "FROM ChatBoot "
            + "WHERE ipaddr = ? AND boottime > ? ";
        try {
            aConn = getConnection();
            aPS = aConn.prepareStatement(aSQL);
            aPS.setString(1, theIP);
            aPS.setLong  (2, System.currentTimeMillis() - Protocol.BOOT_DURATION_MILLIS);
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                return true;
            }
            return false;

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    public boolean isBlockedName ( String theName ) throws SQLException {

        Connection aConn = null;
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT name "
            + "FROM ChatBoot "
            + "WHERE name = ? AND boottime > ? ";
        try {
            aConn = getConnection();
            aPS = aConn.prepareStatement(aSQL);
            aPS.setString(1, theName);
            aPS.setLong  (2, System.currentTimeMillis() - Protocol.BOOT_DURATION_MILLIS);
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                return true;
            }
            return false;

        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    public void recordMessage ( ChatRoom theRoom, 
                                ChatConnection theConn, 
                                String theMessage ) throws SQLException {
        Connection aConn = null;
        PreparedStatement aPS = null;
        String aSQL
            = "INSERT INTO ChatComments ( posted, archid, roomname, fromname, message ) "
            + "VALUES ( ?, ?, ?, ?, ? ) ";
        try {
            aConn = getConnection();
            aPS = aConn.prepareStatement(aSQL);
            int i=1;
            aPS.setLong  (i++, System.currentTimeMillis());
            aPS.setInt   (i++, 1); // For now use default archive ID
            aPS.setString(i++, theRoom.getRoomname());
            aPS.setString(i++, theConn.getUserName());
            aPS.setString(i++, theMessage);
            aPS.executeUpdate();

        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeConnection(aConn);
        }
    }

    private void log ( Exception e ) {
        log(Util.getStackTrace(e));
    }

    private void log ( String s ) {
        itsServer.log("[PStore] " + s);
    }

    private Connection getConnection() throws SQLException {
        return DriverManager.getConnection(itsDBURL, itsDBUser, itsDBPassword);
    }
}
